#!/usr/bin/python3
#  coding: utf-8
#
#   GtkParts.py
#
#       Gtk+3のGUI部品の設定用関数を準備
#
#   20/01/08    新規作成
#


import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gdk, GdkPixbuf


#---------------------
#  comboBoxText class
#---------------------
class comboBoxText:
    """
    comboBoxText用の関数。

    Attribute:
        comboBox (Gtk.Object) :Gtk.ComboBoxTextオブジェクト
    """
    def __init__(self, comboBox):
        self.comboBox = comboBox

    def setItems(self, items):
        """ 
        comboBoxTextにデータをセットする。
        
        Args:
            items (list(str))   :comboBoxにセットするデータ
        Returns:
            None
        """
        self.setData(items)

    def setData(self, items):
        """ 
        comboBoxTextにデータをセットする。
        
        Args:
            items (list(str))   :comboBoxにセットするデータ
        Returns:
            None
        """
        self.comboBox.clear()
        listModel = Gtk.ListStore(str)
        for name in items:
            listModel.append([name])
        cell = Gtk.CellRendererText()
        self.comboBox.pack_start(cell, True)
        self.comboBox.add_attribute(cell, "text", 0)
        self.comboBox.set_model(listModel)

    def getSelectedItem(self):
        """
        選択（active）している項目を取得する。

        Args:
            None
        Returns:
            item (str)  :選択している項目
        """
        return self.getSelectedData()

    def getSelectedData(self):
        """
        選択（active）している項目を取得する。

        Args:
            None
        Returns:
            item (str)  :選択している項目
        """
        entry = self.comboBox.get_child()
        item = entry.get_text()
        return item

    def selectItem(self, selItem):
        """ 
        comboBox内のselItem(int or str)をアクティブに設定する。

        Args:
            selItem (int or str)    :アクティブにするNo or 項目名
        Returns:
            None
        """
        if type(selItem) == int:
            self.comboBox.set_active(selItem)
        else:
            listModel = self.comboBox.get_model()
            for i in range(len(listModel)):
                if listModel[i][0] == selItem:
                    self.comboBox.set_active(i)
                    break


#------------------
#  textView class
#------------------
class textView:
    """
    textViewを扱う場合の関数

    Attribute:
        textView (Gtk.Object)   :Gtk.TextView
    """
    def __init__(self, textView):
        self.textView = textView

    def setText(self, text):
        """ textViewにデータをセットする。
        
        Args:
            text (str) :textViewに表示させるtextデータ
        Returns:
            None
        """
        self.setData(text)

    def setData(self, cont):
        """ textViewにデータをセットする。
        
        Args:
            cont (str) :textViewに表示させるtextデータ
        Returns:
            None
        """
        buffer = self.textView.get_buffer()
        buffer.set_text(cont)

    def getText(self):
        """ textViewからデータを取得する。
        
        Args:
            None
        Returns:
            text (str)  :textView内の全データ
        """
        self.getData()

    def getData(self):
        """ textViewからデータを取得する。
        
        Args:
            None
        Returns:
            text (str)  :textView内の全データ
        """
        buffer = self.textView.get_buffer()
        (start, end) = buffer.get_bounds()
        text = buffer.get_text(start,end,False)
        return text


#-----------------
#  treeList class
#-----------------
class treeList:
    """
    treeViewをtreeListとして扱う時の関数

    Attribute:
        treeList (Gtk.Object)   :Gtk.TreeView
    """
    def __init__(self, treeList):
        self.treeList = treeList

    def create(self, multi=True, header=False, headerCont=""):
        """ treeListの初期化。treeListを作成する。
        
        Args:
            multi (bool)    :MULTIPLE（複数選択）として設定する
            header (bool)   :header(title)行を非表示設定
            headerCont (str):headerの内容
        """
        #multiSelectionの設定
        if multi == True:
            selection = self.treeList.get_selection()
            selection.set_mode(Gtk.SelectionMode.MULTIPLE)
        #titleの設定
        if header == False:
            self.treeList.set_headers_visible(False)
        #treeColumnの設定
        cell = Gtk.CellRendererText()
        col = Gtk.TreeViewColumn(headerCont)
        col.pack_start(cell, True)
        col.add_attribute(cell, "text", 0)
        self.treeList.append_column(col)
        #modelを設定
        listStore = Gtk.ListStore(str)
        self.treeList.set_model(listStore)

    def setItems(self, items):
        """ treeListにデータをセット。
        
        Args:
            items (list(str))   :treeListに表示させるデータ
        Returns:
            None
        """
        self.setData(items)

    def setData(self, items):
        """ treeListにデータをセット。
        
        Args:
            items (list(str))   :treeListに表示させるデータ
        Returns:
            None
        """
        listStore = Gtk.ListStore(str)
        for name in items:
            listStore.append([name])
        self.treeList.set_model(listStore)

    def selectItems(self, selItems):
        """ selItemsを選択する。
        
        Args:
            selItems (list(str))    :選択表示させるデータ
        Returns:
            None
        """
        selection = self.treeList.get_selection()
        listStore = self.treeList.get_model()
        for i in range(len(listStore)):
            item = listStore[i][0]
            if item in selItems:
                path = Gtk.TreePath(i)
                selection.select_path(path)
    
    def selectItemNos(self, selNos):
        """ 指定した項目Noを選択する。
        
        Args:
            selNos (list(int))    :選択表示させる項目No
        Returns:
            None
        """
        selection = self.treeList.get_selection()
        for selNo in selNos:
            path = Gtk.TreePath(selNo)
            selection.select_path(path)

    def getSelectedItems(self):
        """ 選択項目を戻す。
        
        Args:
            None
        Returns:
            selNames (list(str))    :選択している項目
        """
        selection = self.treeList.get_selection()
        (listStore, pathes) = selection.get_selected_rows()
        selNames = []
        for path in pathes:
            iter = listStore.get_iter(path)
            value = listStore.get_value(iter, 0)
            selNames.append(value)
        return selNames
    
    def getAllItems(self):
        """ treeListから全項目を取得する。
        
        Args:
            None
        Returns:
            items (list(str))   :treeList内の全データを取得する
        """
        listStore = self.treeList.get_model()
        items = []
        for i in range(len(listStore)):
            item = listStore[i][0]
            items.append(item)
        return items


#-----------------
#  treeListIcon class
#-----------------
class treeListIcon:
    """
    treeViewをtreeListとして扱う時の関数。
    項目には、アイコンが追加される。
    iconは、pixbufだが、Noneを設定すると、iconが非表示となる。

    Attribute:
        treeList (Gtk.Object)   :Gtk.TreeView
    """
    def __init__(self, treeList):
        self.treeList = treeList

    def create(self, multi=True, header=False, headerCont=""):
        """ treeListの初期化。
        defaultは、複数選択・header非表示。
        
        Args:
            multi (bool)    :MULTIPLE（複数選択）として設定する
            header (bool)   :header(title)行を非表示設定
            headerCont (str):headerの内容
        """
        #multiSelectionの設定
        if multi == True:
            selection = self.treeList.get_selection()
            selection.set_mode(Gtk.SelectionMode.MULTIPLE)
        #titleの設定
        if header == False:
            self.treeList.set_headers_visible(False)
        #modelの設定
        pixbuf = GdkPixbuf.Pixbuf
        listStore = Gtk.ListStore(pixbuf, str)
        self.treeList.set_model(listStore)
        #treeColumnの設定
        icon = Gtk.CellRendererPixbuf()
        cell = Gtk.CellRendererText()
        #cell.set_property("editable", True)
        col = Gtk.TreeViewColumn(headerCont)
        col.pack_start(icon, False)
        col.pack_start(cell, False)
        col.add_attribute(icon, "pixbuf", 0)
        col.add_attribute(cell, "text", 1)
        self.treeList.append_column(col)

    def setItems(self, items):
        """ treeListにデータをセット。
        項目は、[icon, name]のlist形式。
        
        Args:
            items (list(pixbuf, str))   :treeListに表示させるデータ
        """
        self.setData(items)

    def setData(self, items):
        """ treeListにデータをセット。
        項目は、[icon, name]のlist形式。
        
        Args:
            items (list(pixbuf, str))   :treeListに表示させるデータ
        """
        pixbuf = GdkPixbuf.Pixbuf
        listStore = Gtk.ListStore(pixbuf, str)
        for icon, name in items:
            listStore.append([icon, name])
        self.treeList.set_model(listStore)

    def getSelectedItems(self):
        """ 選択項目を戻す。
        
        Returns:
            selNames (list(pixbuf, str))    :選択している項目
        """
        selection = self.treeList.get_selection()
        (listStore, pathes) = selection.get_selected_rows()
        selNames = []
        for path in pathes:
            iter = listStore.get_iter(path)
            icon = listStore.get_value(iter, 0)
            value = listStore.get_value(iter, 1)
            selNames.append([icon, value])
        return selNames

    def unselectAllItems(self):
        """ itemの選択を無くす。
        
        Args:
            None"""
        self.treeList.get_selection().unselect_all()

    def selectItems(self, selItems):
        """ selItemsを追加して選択する。
        
        Args:
            selItems (list(pixbuf, str))    :選択表示させるデータ
        """
        selection = self.treeList.get_selection()
        listStore = self.treeList.get_model()
        for i in range(len(listStore)):
            icon = listStore[i][0]
            name = listStore[i][1]
            if [icon, name] in selItems:
                path = Gtk.TreePath(i)
                selection.select_path(path)
    
    def selectItemNames(self, selNames):
        """ itemを選択する。
        
        Args:
            selNames (list(str))    :選択表示させるitem名"""
        selection = self.treeList.get_selection()
        listStore = self.treeList.get_model()
        for i in range(len(listStore)):
            name = listStore[i][1]
            if name in selNames:
                path = Gtk.TreePath(i)
                selection.select_path(path)

    def getAllItems(self):
        """ treeListから全項目を取得する。
        
        Returns:
            items (list(pixbuf, str))   :treeList内の全データを取得する
        """
        listStore = self.treeList.get_model()
        items = []
        for i in range(len(listStore)):
            icon = listStore[i][0]
            name = listStore[i][1]
            items.append([icon, name])
        return items


#-----------------------
#  treeListCols class
#-----------------------
class treeListCols:
    """
    treeViewをtreeListとして扱う時の関数。
    複数のcolumnを持つtreeList。columnの構成は、modelで設定する。

    Attribute:
        treeList (Gtk.Object)   :Gtk.TreeView
    """

    def __init__(self, treeList):
        self.treeList = treeList

    def create(self, model, multi=True, header=False, headerConts=[], colWidthes=[]):
        """ treeListの初期化。
        defaultは、複数選択・header非表示。
        itemの構成は、modelで定義する。

        Args:
            model (tuple)   :columnの構成を定義
                            :model=(pixbuf, str, pixbuf, str)
            multi (bool)    :MULTIPLE（複数選択）として設定する
            header (bool)   :header(title)行を非表示設定
            headerConts (str):headerの内容（icon含まず）
                            :headerConts=["title", "item"]
            colWidthes (int):列幅 colWidthes=[10, 0]（icon含まず）
        """
        self.model = model
        selection = self.treeList.get_selection()
        #multiSelectionの設定
        if multi == True:
            selection = self.treeList.get_selection()
            selection.set_mode(Gtk.SelectionMode.MULTIPLE)
        #titleの設定
        if header == False:
            self.treeList.set_headers_visible(False)
        #header作成
        pixbuf = GdkPixbuf.Pixbuf
        if len(headerConts) == 0:
            headerConts = ["" for i in range(len(filter(lambda x: x!=pixbuf, model)))]
        #list作成
        icon = Gtk.CellRendererPixbuf()
        cell = Gtk.CellRendererText()
        listStore = Gtk.ListStore(*model)
        self.treeList.set_model(listStore)
        #headerを設定
        i = 0
        for cont in headerConts:
            cell = Gtk.CellRendererText()
            #cell.set_property("editable", True)
            col = Gtk.TreeViewColumn(cont)
            if model[i] == pixbuf:
                col.pack_start(icon, False)
                col.add_attribute(icon, "pixbuf", i)
                i += 1
            col.pack_start(cell, False)
            col.add_attribute(cell, "text", i)
            col.set_resizable(True)
            i += 1
            self.treeList.append_column(col)
        #列幅を設定
        for i in range(len(colWidthes)):
            if i < len(self.model) - 1:
                self.treeList.get_column(i).set_fixed_width(colWidthes[i])

    def setItems(self, items):
        """ treeListにデータをセット。
        項目は、modelで定義した内容を設定する。

        Args:
            items (list(pixbuf, str))   :treeListに表示させるデータ
        """
        listStore = self.treeList.get_model()
        listStore.clear()
        for i in range(len(items)):
            item = items[i]
            listStore.append(item)
        self.treeList.set_model(listStore)
        
    def selectItemNos(self, nums):
        """ 指定されたitemNoを選択する
        
        Args:
            nums (list(int))    :選択するitemNo"""
        selection = self.treeList.get_selection()
        for num in nums:
            path = Gtk.TreePath(num)
            selection.select_path(path)        

    def getSelectedItemNos(self):
        """ 選択項目Noを戻す
        
        Returns:
            selNums (list(int))    :選択している項目No"""
        selection = self.treeList.get_selection()
        (_listStore, pathes) = selection.get_selected_rows()
        selNums = []
        for path in pathes:
            selNo = int(Gtk.TreePath.to_string(path))
            selNums.append(selNo)
        return selNums

    def getSelectedItems(self):
        """ 選択項目を戻す。
        
        Returns:
            selItems (list(pixbuf, str)) :選択している項目
        """
        selection = self.treeList.get_selection()
        (listStore, pathes) = selection.get_selected_rows()
        selItems = []
        for path in pathes:
            selNo = int(Gtk.TreePath.to_string(path))
            item = listStore[selNo]
            selItems.append(item)
        return selItems

    def getSelectedItemNames(self):
        """ 選択項目名のみを戻す。（pixbufは戻さない）
        
        Returns:
            selNames (list(str))    :選択している項目
        """
        selection = self.treeList.get_selection()
        (listStore, pathes) = selection.get_selected_rows()
        selNames = []
        for path in pathes:
            selNo = int(Gtk.TreePath.to_string(path))
            item = listStore[selNo]
            name = []
            for val in item:
                if type(val) == str:
                    #selNames.append(val)
                    name.append(val)
            selNames.append(name)
        return selNames

    def getAllItemNames(self):
        """ treeListからstrの全項目を取得する。"""
        listStore = self.treeList.get_model()
        itemNames = []
        for i in range(len(listStore)):
            item = listStore[i]
            name = []
            for val in item:
                if type(val) == str:
                    name.append(val)
            itemNames.append(name)
        return itemNames

        



            


