#!/usr/bin/python3
#  coding: utf-8
#
#       QtVtk.py
#
#       vtkをQt上で実現させるためのモジュール
#
#   20/05/24    新規作成
#      05/25    getPatchNamesBlockIndex:バグ修正
#               multiRegionで親のpolyMeshが存在しない場合、indexがずれる
#      08/02    getPatchNamesBlockIndex:multiRegionCaseのチェックを追加
#               修正前は、lagrangianがmultiRegionとして設定されてしまう。
#      09/15    vtkのimport状況、vtkのバージョンをを出力。
#      10/29    logFileCreaterのimportを追加。（logを出力させるため）
#   21/06/12    reloadRemakeRenderWindow:再読込用を追加
#


try:
    import logFileCreater
    logFileCreater.log()
except:
    pass

print("importing vtk...")
import vtk
print("  imported " + vtk.vtkVersion.GetVTKSourceVersion())
import QtVtkRenderWindowInteractor as Qvtk
from vtk.util.colors import *
import time
import glob

#from PyQt4 import QtCore, QtGui
import getPyQtModule
PyQtModule = getPyQtModule.PyQtModule

#  import されているPyQtに応じてimportする
if PyQtModule == "PyQt5":
    from PyQt5.QtCore import *
    from PyQt5.QtGui import *
    from PyQt5.QtWidgets import *
elif PyQtModule == "PySide2":
    from PySide2.QtCore import *
    from PySide2.QtGui import *
    from PySide2.QtWidgets import *
elif PyQtModule == "PyQt4":
    from PyQt4.QtCore import *
    from PyQt4.QtGui import *
elif PyQtModule == "PySide":
    from PySide.QtCore import *
    from PySide.QtGui import *
else:
    print("import error: could not import PyQt4, PyQt5, PySide or PySide2!")
    exit()


#--------------------
#  vtkQWidget class
#--------------------
class vtkQWidget(Qvtk.QVTKRenderWindowInteractor):
    """ mouseの操作を「vtkInteractorStyleTrackballCamera()」に設定
    patchのselectSignalを定義
    したものにoverrideする。"""

    #signalを設定
    try:
        #PyQt系
        selectedSignal = pyqtSignal()       #patch選択時
        rightClickSignal = pyqtSignal()     #右クリック時
    except:
        #PySide系
        selectedSignal = Signal()           #patch選択時
        rightClickSignal = Signal()         #右クリック時

    def __init__(self, parent=None):
        super(vtkQWidget, self).__init__(parent)
        #super(vtkQWidget, self).__init__()
        inter = self.GetRenderWindow().GetInteractor()
        inter.SetInteractorStyle(vtk.vtkInteractorStyleTrackballCamera())



#--------------------------
#  OpenFoamPatchMesh class
#--------------------------
class OpenFoamPatchMesh():
    """ OpenFOAMのpatchをvtkを使って3D表示させ、各patchの境界条件を設定する。"""

    #--------------------------
    #  mouseSelectActor class
    #--------------------------
    class mouseSelectActor(vtk.vtkInteractorStyleTrackballCamera):
        """ mouseのeventStyleをoverrideする。"""

        def __init__(self, parent):
            self.AddObserver("LeftButtonPressEvent",self.leftButtonPressEvent)
            self.AddObserver("RightButtonPressEvent",self.rightButtonPressEvent)
            

            self.LastPickedActor = None
            self.LastPickedProperty = vtk.vtkProperty()

            self.actorNo = 0                #選択したactorNo
            self.oldTime = time.time()
            self.parent = parent

        #
        #  leftButtonPressEvent
        #-----------------------
        def leftButtonPressEvent(self, obj, event):
            """ 左クリックのeventをチェック。"""

            clickPos = self.GetInteractor().GetEventPosition()

            picker = vtk.vtkPropPicker()
            picker.Pick(clickPos[0], clickPos[1], 0, self.GetDefaultRenderer())

            # get the new
            self.NewPickedActor = picker.GetActor()

            #doubleClick?
            currTime = time.time()
            dTime = currTime - self.oldTime
            self.oldTime = currTime
            if dTime <= 0.5:
                # If something was selected
                if self.NewPickedActor:
                    # If we picked something before, reset its property
                    #if self.LastPickedActor:
                    #    self.LastPickedActor.GetProperty().DeepCopy(self.LastPickedProperty)
                    
                    # Save the property of the picked actor so that we can
                    # restore it next time
                    #self.LastPickedProperty.DeepCopy(self.NewPickedActor.GetProperty())

                    # Highlight the picked actor by changing its properties
                    # self.NewPickedActor.GetProperty().SetColor(1.0, 0.0, 1.0)
                    # self.NewPickedActor.GetProperty().SetDiffuse(1.0)
                    # self.NewPickedActor.GetProperty().SetSpecular(0.0)
                    
                    # save the last picked actor
                    #self.LastPickedActor = self.NewPickedActor

                    # actorのNoを取得
                    actors = self.GetDefaultRenderer().GetActors()
                    i = 0
                    for actor in actors:
                        if actor == self.NewPickedActor:
                            break
                        i += 1
                    self.actorNo = i

                    #ctrlKeyを確認
                    ctrl, shift = self.parent.vtkWidget._GetCtrlShift(event)
                    if ctrl == True:
                        #actor選択を追加する
                        self.parent.selectAddActor(self.actorNo)
                    else:
                        #actorを選択する
                        self.parent.selectActor(self.actorNo)

            self.OnLeftButtonDown()
            return

        #
        #  rightButtonPressEvent
        #------------------------
        def rightButtonPressEvent(self, obj, event):
            #self.parent.vtkWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
            #self.parent.vtkWidget.setContextMenuPolicy(Qt.CustomContextMenu)
            #self.parent.vtkWidget.customContextMenuRequested.connect(self.parent.showPopupMenu)
            self.parent.vtkWidget.rightClickSignal.emit()
            #self.OnRightButtonDown()
            return


    def __init__(self, vtkWidget, caseDir, region="."):
        self.caseDir = caseDir
        self.fileName = caseDir + "/system/controlDict"
        self.region = region
        self.vtkWidget = vtkWidget  #vtkRenderWindowInteractor
        self.enabled_block = []     #表示する全patch名
        self.selected_block = []    #選択したpatch名
        self.current_block = []     #表示しているpatch名

        self.showEdge = True        #edge表示/非表示flag
        self.setOpacity = False     #透明flag

        self.vtkWidget.setToolTip(_("ダブルクリックしてpatchを選択する"))

        #reader
        self.reader = vtk.vtkOpenFOAMReader()
        self.reader.SetFileName(self.fileName)
        self.reader.Update()
        self.reader.EnableAllPatchArrays()
        self.reader.Update()
        #patch名とblock_indexを取得
        self.getPatchNamesBlockIndex()

    #
    #  isMultiRegion
    def isMultiRegion(self):
        """ multiRegionCaseかどうかをboundaryからチェックする"""
        regMeshDirs = glob.glob(self.caseDir + "/*/*/polyMesh/boundary")
        ans = False
        if len(regMeshDirs) > 0:
            #caseDir内のfolder名をチェック
            for folderDir in regMeshDirs:
                folder = folderDir[len(self.caseDir)+1:].split("/")[0]
                try:
                    float(folder)
                    ans = True
                except:
                    if folder == "constant":
                        ans = True
                if ans == True:
                    break
        return ans

    #
    #  getPatchNamesBlockIndex
    def getPatchNamesBlockIndex(self):
        """
        通常のcaseとmultiRegionCaseでindexNoが異なっている。
        
         通常case                 multiRegionCase
          index   items            index     items
            0 Multi-block Dataset    0 Multi-block Dataset
            1   internalMesh         1   defaultRegion
            2   patches              2     internalMesh
            3     movingWall         3   patches
            4     fixedWalls         4     movingWall
               :                        :
        """
        #全patchNamesとmultiRegionのチェック
        nPatches = self.reader.GetNumberOfPatchArrays()
        allPatchNames = [] 
        multiRegion = 0
        for i in range(nPatches):
            name = self.reader.GetPatchArrayName(i)
            allPatchNames.append(name)
            if len(name.split("/")) > 1:
                #multiRegionの場合、flagを立てる。
                multiRegion = 1
        #親のpolyMeshをチェック
        if len(allPatchNames) > 0:
            if len(allPatchNames[0].split("/")) > 1:
                #multiRegionで親のpolyMeshが存在しない場合
                multiRegion = 2
        #multiRegionの再チェック
        if multiRegion != 0:
            if self.isMultiRegion() == False:
                multiRegion = 0
        #blockIndexを取得
        block_index = {}
        if multiRegion == 0:
            #通常のcaseのindexを取得
            for i in range(nPatches):
                name = allPatchNames[i]
                if name.find("internalMesh") >= 0:
                    #internalMeshの場合
                    index = 1
                    block_index[name] = index
                    index += 1
                else:
                    #patchの場合
                    block_index[name] = index
                index += 1
        else:
            #multiRegionCaseのindexを取得
            index = 1
            if multiRegion == 2:
                #親のployMeshが存在しない場合
                index = 2
            pFlag = 0
            for i in range(nPatches):
                name = allPatchNames[i]
                if name.find("internalMesh") >= 0:
                    index += 1
                    block_index[name] = index
                    pFlag = 0
                else:
                    if pFlag == 0:
                        index += 1
                        block_index[name] = index
                        pFlag = 1
                    else:
                        block_index[name] = index
                index += 1
        #indexの取得結果を出力
        for name in block_index:
            print(name, block_index[name])
        self.block_index = block_index
        #region内の全patch名を取得
        self.enabled_block = []
        if self.region == ".":
            for name in allPatchNames:
                if name.find("internalMesh") < 0:
                    if len(name.split("/")) == 1:
                        self.enabled_block.append(name)
        else:
            for name in allPatchNames:
                if name.find("internalMesh") < 0:
                    words = name.split("/")
                    if len(words) >= 2:
                        if words[0] == self.region:
                            self.enabled_block.append(name)
        self.current_block = self.enabled_block[:]

    #
    #  getPatchNames
    #----------------
    def getPatchNames(self):
        """ region内の全patch名を返す"""
        return self.enabled_block

    #
    #  makeRenderWindow
    #-------------------
    def makeRenderWindow(self):
        """ renderWindowを作成する。
        enabled_block[0]のpatchを選択表示させる。"""
        #patch毎のactorをextract_blockから作成
        self.actorDict = {}
        for name in self.enabled_block:
            #patchを取得
            extract_block = vtk.vtkExtractBlock()
            extract_block.SetInputData(self.reader.GetOutput())
            extract_block.AddIndex(self.block_index[name])
            extract_block.Update()
            #filter
            geom_filter = vtk.vtkGeometryFilter()
            geom_filter.SetInputConnection(extract_block.GetOutputPort())
            # mapper
            mapper = vtk.vtkCompositePolyDataMapper()
            mapper.SetInputConnection(geom_filter.GetOutputPort())
            mapper.ScalarVisibilityOff()
            # actor
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            self.actorDict[name] = actor
        self.current_block = self.enabled_block[:]

        #select表示させる（actorのpropertyを設定）
        #selPatchNames = [self.enabled_block[0]]
        selPatchNames = []
        self.selectPatches(selPatchNames)

        # renderer
        self.ren = vtk.vtkRenderer()
        self.ren.GradientBackgroundOn()
        self.ren.SetBackground(1, 1, 1)
        self.ren.SetBackground2(0.4, 0.55, 0.75)
        #self.ren.SetUseHiddenLineRemoval(True)
        #self.ren.UseDepthPeelingOn()
        #self.ren.UseShadowsOn()
        #self.ren.UseDepthPeelingForVolumesOn()
        #self.ren.UseDepthPeelingOn()
        for name in self.enabled_block:
            self.ren.AddActor(self.actorDict[name])
            #self.ren.AddActor(self.edgeActorDict[name])
        #self.ren.UseShadowsOn()

        renWin = self.vtkWidget.GetRenderWindow()
        renWin.NewInstance()
        renWin.AddRenderer(self.ren)
        #renWin.SetAlphaBitPlanes(False)
        #renWin.SetMultiSamples(0)

        #styleを設定(event)
        style = self.mouseSelectActor(self)
        style.SetDefaultRenderer(self.ren)
        interactor = renWin.GetInteractor()
        interactor.SetInteractorStyle(style)

        #add axis
        self.axesActor = vtk.vtkAxesActor()
        self.axes = vtk.vtkOrientationMarkerWidget()
        self.axes.SetOrientationMarker(self.axesActor)
        self.axes.SetInteractor(interactor)
        self.axes.EnabledOn()
        self.axes.InteractiveOn()
        self.ren.ResetCamera()

    #
    #  deleteAllActors
    #------------------
    def deleteAllActors(self):
        """ 表示されている全てのactorを削除する"""
        #renderWindowから全actorを削除する
        for delActorName in self.actorDict.keys():
            if delActorName in self.current_block:
                self.ren.RemoveActor(self.actorDict[delActorName])
                self.current_block.remove(delActorName)

    #
    #  reloadRemakeRenderWindow
    #---------------------------
    def reloadRemakeRenderWindow(self):
        """ reloadしてrenderWindowを再作成する。"""
        #actorを再作成
        self.actorDict = {}
        for name in self.enabled_block:
            #patchを取得
            extract_block = vtk.vtkExtractBlock()
            extract_block.SetInputData(self.reader.GetOutput())
            extract_block.AddIndex(self.block_index[name])
            extract_block.Update()
            #filter
            geom_filter = vtk.vtkGeometryFilter()
            geom_filter.SetInputConnection(extract_block.GetOutputPort())
            # mapper
            mapper = vtk.vtkCompositePolyDataMapper()
            mapper.SetInputConnection(geom_filter.GetOutputPort())
            mapper.ScalarVisibilityOff()
            # actor
            actor = vtk.vtkActor()
            actor.SetMapper(mapper)
            self.actorDict[name] = actor
        self.current_block = self.enabled_block[:]
        #select表示させる（actorのpropertyを設定）
        selPatchNames = []
        self.selectPatches(selPatchNames)
        #renderWindowにactorを追加
        for name in self.enabled_block:
            self.ren.AddActor(self.actorDict[name])
        #表示させる
        self.vtkWidget.Initialize()

    #
    #  selectActor
    #---------------
    def selectActor(self, actorNo):
        """ mouseのdoubleClickで選択したactorを選択表示に設定する。
        
        Args:
            actorNo (int)   :actorのindexNo"""
        selPatch = self.current_block[actorNo]
        selPatchNames = [selPatch]
        self.selectPatches(selPatchNames)

    #
    #  selectAddActor
    #-----------------
    def selectAddActor(self, actorNo):
        """ mouseのdoubleClickで選択したactorを選択表示に追加設定する。
        
        Args:
            actorNo (int)   :actorのindexNo"""
        selPatch = self.current_block[actorNo]
        selPatchNames = self.selected_block[:]
        selPatchNames.append(selPatch)
        self.selectPatches(selPatchNames)

    #
    #  selectPatches
    #----------------
    def selectPatches(self, selPatchNames):
        """
        指定したpatchを選択表示、それ以外を非選択表示に設定する。
        
        Args:
            selPatchNames (list(str))   :選択表示させるpatch名
        """
        for name in self.enabled_block:
            #prop = self.actorDict[name].GetProperty()
            if name in selPatchNames:
                #選択部
                prop = self.actorDict[name].GetProperty()
                prop.BackfaceCullingOn()
                prop.FrontfaceCullingOn()
                prop.SetAmbient(0.5)
                prop.SetColor(red)
                if self.showEdge == True:
                    prop.EdgeVisibilityOn()
                else:
                    prop.EdgeVisibilityOff()
                if self.setOpacity == True:
                    prop.SetOpacity(0.5)
                else:
                    prop.SetOpacity(1.0)
                prop.SetEdgeColor(white)
                prop.SetRepresentationToSurface()
            else:
                #非選択部
                prop = self.actorDict[name].GetProperty()
                prop.BackfaceCullingOn()
                prop.FrontfaceCullingOn()
                prop.SetAmbient(0.3)
                prop.SetColor(green)
                if self.showEdge == True:
                    prop.EdgeVisibilityOn()
                else:
                    prop.EdgeVisibilityOff()
                #prop.EdgeVisibilityOff()
                if self.setOpacity == True:
                    prop.SetOpacity(0.5)
                else:
                    prop.SetOpacity(1.0)
                prop.SetEdgeColor(black)
                prop.SetRepresentationToSurface()
        self.selected_block = selPatchNames
        self.vtkWidget.selectedSignal.emit()

    #
    #  remakeRenderWindow
    #---------------------
    def remakeRenderWindow(self):
        """
        隠したpatchを表示し、元の形状を表示する。
        """
        #全actorを削除
        for actorName in self.current_block:
            self.ren.RemoveActor(self.actorDict[actorName])
        #全actorを表示
        for actorName in self.enabled_block:
            self.ren.AddActor(self.actorDict[actorName])
        self.current_block = self.enabled_block[:]
        # addPatches = list(set(self.enabled_block).difference(self.current_block))
        # for patch in addPatches:
        #     self.ren.AddActor(self.actorDict[patch])
        #     self.current_block.append(patch)

    #
    #  hidePatches
    #---------------
    def hidePatches(self, hidePatchNames):
        """ 指定したpatchを隠し、表示させない。
        """
        if len(self.current_block) <= 1:
            return
        for hideName in hidePatchNames:
            if hideName in self.current_block:
                self.ren.RemoveActor(self.actorDict[hideName])
                self.current_block.remove(hideName)

    #
    #  showPopupMenu
    #-----------------
    def showPopupMenu(self, pos):
        """ popupMenuを表示する。"""
        #menu = QtGui.QMenu()
        menu = QMenu()
        menu.addAction(_("edge表示/非表示の切替え"), self.onSwitchShowEdge)
        menu.addAction(_("透明/不透明の切替え"), self.onSwitchOpacity)
        menu.addAction(_("選択patchを隠す"), self.onHideSelectedPatches)
        menu.addAction(_("全patchを表示"), self.onShowAllPatches)
        menu.exec_(self.vtkWidget.mapToGlobal(pos))

    def onSwitchShowEdge(self):
        if self.showEdge == True:
            self.showEdge = False
        else:
            self.showEdge = True
        selectPatchNames = self.selected_block
        self.selectPatches(selectPatchNames)

    def onSwitchOpacity(self):
        if self.setOpacity == True:
            self.setOpacity = False
        else:
            self.setOpacity = True
        selectPatchNames = self.selected_block
        self.selectPatches(selectPatchNames)

    def onHideSelectedPatches(self):
        selectedPatches = self.selected_block
        self.hidePatches(selectedPatches)

    def onShowAllPatches(self):
        self.remakeRenderWindow()


if __name__ == "__main__":
    import gettext
    gettext.install("app")
    #_ = gettext.gettext
