#!/usr/bin/python3
# coding: utf-8
#
#   editStlFilesDialog.py
#
#       stlファイルのsolid名や倍率、face向き等を変更する
#
#   13/10/31    新規作成
#      11/02    binaryの場合は、solid名、サイズとも空白で表示
#      11/03    binaryの場合も表示、ascii変換追加
#      12/21    stdout、stderrの設定（import logFileCreater）を追加
#   14/12/27    solid名の取得、変更方法を修正、stl結合ボタンを追加
#   15/01/10    mergeStlFile:バグ修正（stl結合時、cancelできない）
#      01/13    「face向き反転」「solid名設定」「stl内部結合」「folder開く」ﾎﾞﾀﾝ追加
#               toolTip追加、ﾀﾞﾌﾞﾙｸﾘｯｸの動作追加、「paraview起動」ボタン追加
#      01/15    setSolidName:バグ修正
#      06/27    windowのサイズをwxのバージョンによって方法を変える
#      07/14    winSizeの保存場所変更
#      07/17    dialog起動方法修正
#   16/05/09    binaryの読み込み方法を修正
#      08/28    windowサイズ設定を修正
#   19/02/01    windowサイズ設定を修正(wx.vrsion()のgtkVerを追加）
#               setSolidName:stl内のsolid名に「,」が含まれる事があるので、これに対応
#      05/21    copyFileToTriSurface:「mkdir」を「os.mkdir」に修正
#      09/13    GTK+3, python3用に大幅修正
#   20/04/21    多言語化対応
#      06/02    copyStlFiles, pasteStlFiles:copyPasteFolders.pyに変更。
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, Gdk, GdkPixbuf
import os, sys
import glob
import struct
import math
import shutil
import threading
import time
import decimal

import pyTreeFoam
import universalDialogs as unvDlg
import inputTextDDialog
import GtkParts
import copyPasteFolders

import locale
localeDir = os.getenv("TreeFoamPath") + "/data/locale"
locale.bindtextdomain("treefoam", localeDir)

import logFileCreater
logFileCreater.log()

#------------
#  windowApp
#------------
class windowApp:

    def __init__(self, caseDir, stlDir):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path+"editStlFilesDialog.glade")
        self.mainWindow = self.builder.get_object("window1")
        self.mainWindow.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        #GUIのobject名を取得
        self.setGtkObject()
        #変数の設定
        self.caseDir = caseDir
        self.stlDir = stlDir
        self.clipboardFile = os.getenv("TreeFoamUserPath") + "/data/clipBoard"
        #iconの設定
        iconDir = os.getenv("TreeFoamPath") + "/icons"
        iconFile = GdkPixbuf.Pixbuf.new_from_file
        self.iconFile = iconFile(iconDir + "/text8.png")
        self.iconFileLink = iconFile(iconDir + "/textLink8.png")
        #popupMenuItemsを設定
        self.setPopupMenuItems()
        #初期化
        self.initialize()

    def main(self):
        """ GUIの表示"""
        self.mainWindow.show()
        Gtk.main()

    def close(self, *args):
        """ 閉じる"""
        Gtk.main_quit()

    def setGtkObject(self):
        """ glade内のobject名を取得"""
        self.treeView = self.builder.get_object("treeView")

    #----------- event handler -------
    #ascii変換
    def onBinToAscii(self, event):
        self.binToAscii()
    #solid名セット
    def onSetSolidName(self, event):
        self.setSolidName()
    #solid名変更
    def onChangeSolidName(self, event):
        self.changeSolidName()
    #scale変更
    def onChangeScale(self, event):
        self.changeScale()
    #faceの向きを反転
    def onReverseSurface(self, event):
        self.reverseSurface()
    #stlFile結合
    def onMergeStlFiles(self, event):
        self.mergeStlFile()
    #内部solid名削除（内部結合）
    def onDeleteInnerNames(self, event):
        self.deleteInnerNames()
    #paraview起動
    def onRunParaview(self, event):
        self.runParaview()
    #folder開く
    def onOpenFolder(self, event):
        self.openFolder()
    #閉じる
    def onClose(self, event):
        #ending(stlDir、winSizeを保存)
        self.ending()
        self.close()
    #-------- mouse操作------
    #pressButton
    def onPressMouseButton(self, widget, event):
        self.pressMouseButton(widget, event)
    #-------- popupMenu ------------
    #開く
    def onOpenStlFiles(self, event):
        self.openStlFiles()
    #コピー
    def onCopyStlFiles(self, event):
        self.copyStlFiles()
    #貼り付け
    def onPasteStlFiles(self, event):
        self.pasteStlFiles()
    #ファイル名変更
    def onRenameStlFile(self, event):
        self.renameStlFile()
    #削除
    def onDeleteStlFiles(self, event):
        self.deleteStlFiles()


    #
    #  setPopupMenuItems
    #--------------------
    def setPopupMenuItems(self):
        """ popupMenu項目を設定する"""
        self.popupMenuItems = [
            #ラベル                     handler
            [_("開く"),                 self.onOpenStlFiles],
            ["-", ""],
            [_("コピー"),               self.onCopyStlFiles],
            [_("フォルダに貼付け"),     self.onPasteStlFiles],
            ["-", ""],
            [_("ファイル名変更"),       self.onRenameStlFile],
            [_("削除"),                 self.onDeleteStlFiles]
            ]

    #
    #  initialize
    #--------------
    def initialize(self):
        """ 初期化"""
        #configDictを取得
        self.configDict = pyTreeFoam.readConfigTreeFoam()
        #windowSizeを取得
        winSizeDict = pyTreeFoam.readWindowSize()
        #windowSizeを設定
        xy = winSizeDict["editStlFiles"]
        if xy != "":
            (x, y) = map(int, xy.split()[:2])
            self.mainWindow.set_default_size(x, y)
        #treeViewのcolumnWidthを取得
        colWidthes = winSizeDict["editStlFiles_colWidth"]
        if colWidthes != "":
            widthes = list(map(int, colWidthes.split()))
        else:
            widthes = []
        #treeListを作成
        pixbuf = GdkPixbuf.Pixbuf
        model = (pixbuf, str, str, str, str)
        header = ["stl", "format", "solid", "size(xyz)"]
        treeList = GtkParts.treeListCols(self.treeView)
        treeList.create(model, 
                multi=True, 
                header=True, 
                headerConts=header,
                colWidthes=widthes)
        #tree表示
        self.setDataToListBox()

    #
    #  ending
    #
    def ending(self):
        """ 終了処理。stlDirとwinSizeを保存する"""
        #stlDirを保存
        dataDict = {}
        dataDict["stlDir"] = self.stlDir
        pyTreeFoam.case(self.caseDir).setCreateMeshData(dataDict)
        #winSizeを保存
        (width, height) = self.mainWindow.get_size()
        colWidthes = []
        for i in range(3):
            colWidth = self.treeView.get_column(i).get_width()
            colWidthes.append(str(colWidth))
        winDict = {}
        winDict["editStlFiles"] = str(width) + " " + str(height)
        winDict["editStlFiles_colWidth"] = " ".join(colWidthes)
        pyTreeFoam.writeWindowSize(winDict)

    #
    #  binToAscii
    #  ----------
    def binToAscii(self):
        """ 選択ファイルをascii形式に変換する"""
        stlNames = self.getSelectStlNames()
        if len(stlNames) == 0:
            title = _("エラー")
            mess = _("stlファイルを選択してください")
            self.errDialog(title, mess)
            return
        #倍率を変更(asciiに変換)
        envOpenFoam = self.configDict["bashrcFOAM"]
        for name in stlNames:
            fileName = self.stlDir + os.sep + name
            comm = ". " + envOpenFoam + "; "
            comm += "surfaceTransformPoints -scale '(1.0 1.0 1.0)' " + fileName + " " + fileName
            pyTreeFoam.run(self.caseDir).commandWithLog(comm)
        #fileをtriSurfaceにコピー
        self.copyFilesToTriSurface(stlNames)
        #listBoxに反映
        self.setDataToListBox()
        #変換したfileを選択する
        self.selectStlNames(stlNames)
        print(_("ascii形式に変換しました。"))

    #
    #  setSolidName
    #  ------------
    def setSolidName(self):
        """ file名のsolid名をセットする"""
        stlNames = self.getSelectStlNames()
        if len(stlNames) == 0:
            title = _("エラー")
            mess = _("stlファイルを選択してください")
            self.errDialog(title, mess)
            return
        #asciiチェック
        flag = 0
        for stlName in stlNames:
            fileName = stlDir + "/" + stlName
            if isBinaryStl == True:
                flag = 1
                break
        if flag == 1:
            title = _("エラー")
            mess = _("選択したstlファイルは、binaryが含まれています。")
            mess += _("ascii形式に変換後、実行してください。")
            self.errDialog(title, mess)
            return
        #solid名をセット
        for stlName in stlNames:
            flag = 0
            fileName = stlDir + "/" + stlName
            name = stlName[:-4]
            f=open(fileName); lines=f.readlines(); f.close()
            #旧のsolid名を取得
            words = lines[0].split()
            oldName = ""
            if len(words) > 1:
                oldName = words[1]
                if oldName.find(",") >= 0:
                    #1行目のnameに「,」が含まれているかどうか
                    oldName = oldName.split(",")[0]
            #solid名を設定
            lines[0] = " ".join([words[0], name]) + "\n"
            i = len(lines) - 1
            while i > 0:
                words = lines[i].split()
                if words[0] == "endsolid":
                    endName = ""
                    if len(words) > 1:
                        endName = words[1]
                    break
                i -= 1
            if oldName != endName:
                flag += 1
            else:
                lines[i] = " ".join([words[0], name]) + "\n"
                f=open(fileName, "w")
                for line in lines:
                    f.write(line)
                f.close()
        #fileをtriSurfaceにコピー
        self.copyFilesToTriSurface(stlNames)
        #listBoxに反映
        self.setDataToListBox()
        #変換したfileを選択する
        self.selectStlNames(stlNames)
        print(_("solid名をセットしました。"))

    #
    #  changeSolidName
    #  ---------------
    def changeSolidName(self):
        """ 選択しているstlファイルのsolid名を変更する"""
        treeList = GtkParts.treeListCols(self.treeView)
        stlNames = treeList.getSelectedItemNames()
        #エラーチェック
        if len(stlNames) == 0:
            title = _("エラー")
            mess = _("stlファイルを選択してください")
            self.errDialog(title, mess)
            return
        if len(stlNames) != 1:
            title = _("エラー")
            mess = _("複数のstlファイルを選択しています。")
            mess += _("\n1ヶのstlファイルを選択してください。")
            self.errDialog(title, mess)
            return
        #変更するsolidNameを取得
        solidName = stlNames[0][2]
        title = _("solid名の入力")
        mess = _("新しいsolid名を入力してください")
        newName = self.inputTextDialog(title, mess, solidName)
        if newName == "":
            return
        #solid名を変更
        oldName = " ".join(solidName.split())
        changeFlag = 0
        selName = stlNames[0][0]
        fileName = stlDir + "/" + selName
        #fileFormatをチェック
        if isBinaryStl(fileName) == False:
            #asciiの場合
            f=open(fileName); lines=f.readlines(); f.close()
            i = 0
            for line in lines:
                if line[:5] == "solid":
                    if " ".join(line.split()[1:]) == oldName:
                        lines[i] = "solid " + newName + "\n"
                        changeFlag += 1
                if line[:8] == "endsolid":
                    if changeFlag == 1:
                        lines[i] = "endsolid " + newName + "\n"
                        changeFlag += 1
                i += 1
            if changeFlag == 2:
                f=open(fileName, "w")
                for line in lines:
                    f.write(line)
                f.close()
                print(_("solid名を変更しました。"), solidName, "->", newName)
            else:
                title = _("エラー")
                mess = _("solid名が変更できませんでした。")
                mess += _("\nstlファイルの書式を確認してください。")
                self.errDialog(title, mess)
        else:
            #binaryの場合
            f=open(fileName, "rb"); cont=f.read(); f.close()
            header = newName.encode("utf-8") + b"\00"*(80-len(newName))
            cont = header + cont[80:]
            f=open(fileName, "wb"); f.write(cont); f.close()
            print(_("solid名を変更しました。"), solidName, "->", newName)
        #fileをtriSurfaceにコピー
        self.copyFilesToTriSurface([selName])
        #listBoxに反映
        self.setDataToListBox()
        #変換したfileを選択する
        self.selectStlNames([selName])

    #
    #  changeScale
    #  -----------
    def changeScale(self):
        """ scaleの変更"""
        stlNames = self.getSelectStlNames()
        #エラーチェック
        if len(stlNames) == 0:
            title = _("エラー")
            mess = _("stlファイルを選択してください")
            self.errDialog(title, mess)
            return
        #変更する倍率を取得
        title = _("倍率の入力")
        mess = _("変更する倍率を入力してください")
        scale = self.inputTextDialog(title, mess, "1.0")
        if scale == "":
            return
        #倍率を変更
        envOpenFoam = self.configDict["bashrcFOAM"]
        for name in stlNames:
            fileName = self.stlDir + os.sep + name
            comm = ". " + envOpenFoam + "; "
            comm += "surfaceTransformPoints -scale '(" + scale + " " + scale + " " + scale + ")' " + fileName + " " + fileName
            pyTreeFoam.run(self.caseDir).commandWithLog(comm)
        #fileをtriSurfaceにコピー
        self.copyFilesToTriSurface(stlNames)
        #listBoxに反映
        self.setDataToListBox()
        #変換したfileを選択する
        self.selectStlNames(stlNames)
        print(_("scaleを変更しました。"), " x", scale)

    #
    #  reverseSurface
    #  --------------
    def reverseSurface(self):
        """ faceの向きを反転する"""
        stlNames = self.getSelectStlNames()
        #エラーチェック
        if len(stlNames) == 0:
            title = _("エラー")
            mess = _("stlファイルを選択してください")
            self.errDialog(title, mess)
            return
        #asciiチェック
        flag = 0
        for stlName in stlNames:
            fileName = stlDir + "/" + stlName
            if isBinaryStl == True:
                flag = 1
                break
        if flag == 1:
            title = _("エラー")
            mess = _("選択したstlファイルは、binaryが含まれています。")
            mess += _("ascii形式に変換後、実行してください。")
            self.errDialog(title, mess)
            return
        #faceの向き反転
        newNames = []
        for stlName in stlNames:
            fileName = self.stlDir + "/" + stlName
            f = open(fileName); lines = f.readlines(); f.close()
            i = 0
            while i < len(lines):
                words = lines[i].split()
                if words[0] == "facet" and words[1]== "normal":
                    #ベクトルを反転
                    words[2] = str(-float(words[2]))
                    words[3] = str(-float(words[3]))
                    words[4] = str(-float(words[4]))
                    lines[i] = " " + " ".join(words) + "\n"
                    #三角形の座標の順番を入れ替え
                    point1 = lines[i+3]
                    point2 = lines[i+4]
                    lines[i+3] = point2
                    lines[i+4] = point1
                    i += 5
                else:
                    i += 1
            newFileName = fileName[:-4] + "_rev.stl"
            f = open(newFileName, "w"); f.writelines(lines); f.close()
            newName = newFileName.split(os.sep)[-1]
            newNames.append(newName)
        #fileをtriSurfaceにコピー
        self.copyFilesToTriSurface(newNames)
        #listBoxに反映
        self.setDataToListBox()
        #変換したfileを選択する
        self.selectStlNames(newNames)
        print(_("faceの向きを反転させました。"))

    #
    #  mergeStlFile
    #  ------------
    def mergeStlFile(self):
        """ 選択した複数のstlFileを結合する"""
        stlNames = self.getSelectStlNames()
        #エラーチェック
        if len(stlNames) < 2:
            title = _("エラー")
            mess = _("stlファイルを複数選択してください")
            self.errDialog(title, mess)
            return
        #asciiチェック
        flag = 0
        for stlName in stlNames:
            fileName = stlDir + "/" + stlName
            if isBinaryStl == True:
                flag = 1
                break
        if flag == 1:
            title = _("エラー")
            mess = _("選択したstlファイルは、binaryが含まれています。")
            mess += _("ascii形式に変換後、実行してください。")
            self.errDialog(title, mess)
            return
        #solid名のチェック
        solids = []
        for stlName in stlNames:
            names = []
            fileName = self.stlDir + os.sep + stlName
            f = open(fileName); lines = f.readlines(); f.close()
            for line in lines:
                if line[:5] == "solid":
                    name = " ".join(line.split()[1:])
                    names.append(name)
            solids += names
        flag = 0
        solids.sort()
        oldName = ""
        for solid in solids:
            if len(solid.split()) == 0:
                flag = 1
                break
            elif oldName == solid:
                flag = 2
                break
            oldName = solid
        if flag == 1:
            title = _("エラー")
            mess = _("solid名が設定されていないファイルがあります")
            mess += _("\nstlファイルが結合できません")
            self.errDialog(title, mess)
            return
        if flag == 2:
            title = _("エラー")
            mess = _("同じsolid名が使われているファイルがあります。")
            mess += _("\nstlファイルが結合できません")
            self.errDialog(title, mess)
            return
        #defaultのnewStl名を設定
        allStlNames = glob.glob(self.stlDir + os.sep + "*.stl")
        allStlNames = list(map(lambda x: x.split(os.sep)[-1], allStlNames))
        stlNo = 0
        loop = True
        while loop == True:
            newName = "newStl_" + str(stlNo) + ".stl"
            if newName in allStlNames:
                stlNo += 1
            else:
                loop = False
                break
        #newStl名を取得
        title = _(u"stlファイルの結合")
        mess = _(u"新しいstlファイル名を入力してください")
        inputText = newName
        text = self.inputTextDialog(title, mess, inputText)
        if text == "":
            return
        if text.split(".")[-1] != "stl":
            text += ".stl"
        if text in allStlNames:
            title = _("エラー")
            mess = _("同じファイル名が存在します。")
            mess += _("\nファイル名を変更してください。")
            self.errDialog(title, mess)
            return
        writeName = self.stlDir + os.sep + text
        #stlFileを結合
        for stlName in stlNames:
            readName = self.stlDir + os.sep + stlName
            f = open(readName); cont = f.read(); f.close()
            if cont[-1] != "\n":
                cont += "\n"
            f = open(writeName, "a"); f.write(cont); f.close()
        title = _("stlファイルの結合")
        mess = _("stlファイルを結合しました。")
        self.okDialog(title, mess)
        #fileをtriSurfaceにコピー
        self.copyFilesToTriSurface([text])
        #listBoxに反映
        self.setDataToListBox()
        #変換したfileを選択する
        self.selectStlNames([text])
        print(_("stlファイルを結合しました。"))

    #
    #  deleteInnerNames
    #  ----------------
    def deleteInnerNames(self):
        """ stlファイル内の内部solid名を削除する"""
        stlNames = self.getSelectStlNames()
        #エラーチェック
        if len(stlNames) == 0:
            title = _("エラー")
            mess = _("stlファイルを選択してください")
            self.errDialog(title, mess)
            return
        #asciiチェック
        flag = 0
        for stlName in stlNames:
            fileName = stlDir + "/" + stlName
            if isBinaryStl == True:
                flag = 1
                break
        if flag == 1:
            title = _("エラー")
            mess = _("選択したstlファイルは、binaryが含まれています。")
            mess += _("ascii形式に変換後、実行してください。")
            self.errDialog(title, mess)
            return
        #内部solid名を削除する
        for stlName in stlNames:
            fileName = self.stlDir + os.sep + stlName
            f = open(fileName); lines = f.readlines(); f.close()
            solidName = stlName[:-4]
            #header部以外を取得
            newLines = []
            for line in lines:
                words = line.split()
                if len(words) > 0:
                    name = words[0].lower()
                    if name != "solid" and name != "endsolid":
                        newLines.append(line)
            #header, footerを追加
            header = "solid " + solidName + "\n"
            footer = "endsolid " + solidName + "\n"
            if newLines[-1][-1] != "\n":
                newLines[-1] += "\n"
            newLines = [header] + newLines + [footer]
            f = open(fileName, "w"); f.writelines(newLines); f.close()
        #fileをtriSurfaceにコピー
        self.copyFilesToTriSurface(stlNames)
        #listBoxに反映
        self.setDataToListBox()
        #変換したfileを選択する
        self.selectStlNames(stlNames)
        print(_("内部solid名を削除しました。"))

    #
    #  runParaview
    #  -----------
    def runParaview(self):
        """ paraViewを起動する"""
        print(_("paraviewを起動します"))
        envOpenFoam = self.configDict["bashrcFOAM"]
        comm = ". " + envOpenFoam + "; "
        comm += "paraview &"
        pyTreeFoam.run(self.stlDir).commandWithLog(comm)

    #
    #  openFolder
    #  ----------
    def openFolder(self):
        """ folderを開く"""
        print(_("folderを開く"))
        openDir = self.stlDir
        fileManager = pyTreeFoam.fileManager
        comm = fileManager + " " + openDir + " &"
        pyTreeFoam.run(self.stlDir).command(comm)

    #
    #  pressMouseButton
    #-------------------
    def pressMouseButton(self, widget, event):
        """ 右クリックの動作"""
        pos = (event.x, event.y)
        #rightClick?
        if event.button == 3:
            treeList = GtkParts.treeListCols(self.treeView)
            selNos = treeList.getSelectedItemNos()
            #選択が反転するので、0.1s後に再度選択する
            thread = threading.Thread(target=self.afterSelectRow, args=(selNos,))
            thread.start()
            #popupMenuを表示
            self.showPopupMenu(pos)

    def showPopupMenu(self, pos):
        """ popupMenuを表示する"""
        #popupMenu作成
        menu = Gtk.Menu()
        menuItems = self.popupMenuItems
        for i in range(len(menuItems)):
            menuName = menuItems[i]
            if menuName[0] == "-":
                #セパレータをセット
                item = Gtk.SeparatorMenuItem()
            else:
                item = Gtk.MenuItem()
                item.set_label(menuName[0])
                item.connect("activate", menuName[1])
            menu.append(item)
        #menuを表示
        menu.show_all()
        menu.popup(None, None, None, None, 0, Gtk.get_current_event_time())
    def afterSelectRow(self, selNos):
        """ 0.1s後に選択する"""
        time.sleep(0.1)
        treeList = GtkParts.treeListCols(self.treeView)
        treeList.selectItemNos(selNos)

    #
    #  openStlFiles
    #--------------
    def openStlFiles(self):
        """ stlFileをeditorで開く"""
        treeList = GtkParts.treeListCols(self.treeView)
        selNames = treeList.getSelectedItemNames()
        openNames = list(map(lambda x: x[0], selNames))
        openFiles = list(map(lambda x: self.stlDir + "/" + x, openNames))
        pyTreeFoam.run(self.stlDir).editor(openFiles)

    #
    #  copyStlFiles
    #---------------
    def copyStlFiles(self):
        """ stlFileをclipboardにコピーする"""
        treeList = GtkParts.treeListCols(self.treeView)
        selNames = treeList.getSelectedItemNames()
        copyNames = list(map(lambda x: x[0], selNames))
        copyFiles = list(map(lambda x: self.stlDir + "/" + x, copyNames))
        f = open(self.clipboardFile, "w")
        f.write("foldersCopy\n")
        for fileName in copyFiles:
            print("copy--->", fileName)
            f.write(fileName + "\n")
        f.close()
        # comm = "./copyToSystemClipBoard.py " + self.clipboardFile + " &"
        # path = os.getenv("TreeFoamPath") + "/python"
        # pyTreeFoam.run(path).commandWithLog(comm)
        copyPasteFolders.copyFolders(copyFiles)

    #
    #  pasteStlFiles
    #----------------
    def pasteStlFiles(self):
        """ stlFileを貼り付ける"""
        #clipboardの内容を取得
        copyDirs = copyPasteFolders.pasteFolders()
        # pasteContFile = os.getenv("TreeFoamUserPath") + "/data" + os.sep + "pasteNames"
        # comm = "./pasteFromSystemClipBoard.py " + pasteContFile
        # path = os.getenv("TreeFoamPath") + "/python"
        # pyTreeFoam.run(path).commandWithLog(comm)
        # f = open(pasteContFile); cont = f.read(); f.close()
        # copyDirs = cont.split()
        pasteDir = self.stlDir
        pasteNames = []        
        for copyDir in copyDirs:
            name = os.path.basename(copyDir)
            if len(glob.glob(self.stlDir + "/" + name)) != 0:
                #同じfile名が存在する場合
                #file名を修正する
                i = 0
                pasteFileName = pasteDir + "/" + name[:-4] + "_copy" + str(i) + ".stl"
                while len(glob.glob(pasteFileName)) == 1:
                    i += 1
                    pasteFileName = pasteDir + "/" + name[:-4] + "_copy" + str(i) + ".stl"
                #コピー
                shutil.copy(copyDir, pasteFileName)
                pasteNames.append(os.path.basename(pasteFileName))
        #tree再表示
        self.setDataToListBox()
        #選択表示する。
        treeList = GtkParts.treeListCols(self.treeView)
        allNames = treeList.getAllItemNames()
        selNos = []
        for i in range(len(allNames)):
            words = allNames[i]
            if words[0] in pasteNames:
                selNos.append(i)
        treeList.selectItemNos(selNos)

    #
    #  renameStlFile
    #----------------
    def renameStlFile(self):
        """ stlFile名の変更"""
        treeList = GtkParts.treeListCols(self.treeView)
        selNames = treeList.getSelectedItemNames()
        if len(selNames) >= 2:
            title = _("エラー")
            mess = _("複数選択しています。") + "\n"
            mess += _("1ヶのファイルを選択してください。")
            self.errDialog(title, mess)
            return
        #新しいファイル名を取得
        selName = selNames[0][0]
        title = _("ファイル名の変更")
        mess = _("新しいファイル名を入力してください。")
        defText = selName
        dialog = inputTextDDialog.dialogApp(title, mess, defText)
        newName = dialog.show()
        if newName == "":
            return
        #renameする
        if newName.split(".")[-1] != "stl":
            newName += ".stl"
        newNamePath = self.stlDir + "/" + newName
        selNamePath = self.stlDir + "/" + selName
        os.rename(selNamePath, newNamePath)
        #treeListを再表示
        self.setDataToListBox()
        #選択表示させる
        treeList = GtkParts.treeListCols(self.treeView)
        allNames = treeList.getAllItemNames()
        selNo = 0
        for i in range(len(allNames)):
            words = allNames[i]
            if words[0] == newName:
                selNo = i
                break
        treeList.selectItemNos([selNo])

    #
    #  deleteStlFiles
    #-----------------
    def deleteStlFiles(self):
        """ 選択したstlFileを削除する"""
        treeList = GtkParts.treeListCols(self.treeView)
        selNames = treeList.getSelectedItemNames()
        title = _("stlFileの削除")
        mess = _("以下のファイルを削除します。")
        files = ""
        for selName in selNames:
            files += selName[0] + ", "
            if len(files) > 50:
                files = files + "..."
                break
        if len(files) <= 50:
            files = files[:-2]
        mess += "\n  " + files
        stat = self.okCancelDialog(title, mess)
        if stat != "OK":
            return
        #削除する
        for selName in selNames:
            namePath = self.stlDir + "/" + selName[0]
            os.remove(namePath)
        #再表示させる
        self.setDataToListBox()
        title = _("stlFileの削除")
        mess = _("「") + str(len(selNames)) + _("」ヶのファイルを削除しました。")
        self.okDialog(title, mess)


    #  copyFilesToTriSurface
    def copyFilesToTriSurface(self, stlNames):
        """ stlFileをtiSurfaceにコピーする"""
        folder = self.caseDir + "/constant/triSurface"
        if len(glob.glob(folder)) == 0:
            os.mkdir(folder)
        for name in stlNames:
            copyName = self.stlDir + os.sep + name
            pasteName = folder + os.sep + name
            shutil.copy(copyName, pasteName)

    #  setDataToListBox
    def setDataToListBox(self):
        """ TreeViewにdataをセットし直す"""
        stlFiles = glob.glob(self.stlDir + os.sep + "*.stl")
        stlConts = getStlFilesCont(stlFiles)
        #itemを作成
        items = []
        for stlCont in stlConts:
            stl = stlCont[0]
            stlPath = self.stlDir + "/" + stl
            icon = self.iconFile
            if os.path.islink(stlPath) == True:
                icon = self.iconFileLink
            fmt = stlCont[1]
            solid = stlCont[2]
            size = "  ".join(map(str, stlCont[3]))
            items.append([icon, stl, fmt, solid, size])
        #treeListにitemをセット
        treeList = GtkParts.treeListCols(self.treeView)
        treeList.setItems(items)        

    #  selectStlNames
    def selectStlNames(self, stlNames):
        """ 指定stlを選択表示する"""
        treeList = GtkParts.treeListCols(self.treeView)
        allNames = treeList.getAllItemNames()
        selNos = []
        for i in range(len(allNames)):
            if allNames[i][0] in stlNames:
                selNos.append(i)
        treeList.selectItemNos(selNos)

    #  getSelectStlNames
    def getSelectStlNames(self):
        """ listBoxから選択しているstlNameを取得する"""
        treeList = GtkParts.treeListCols(self.treeView)
        selNames = treeList.getSelectedItemNames()
        selNames = list(map(lambda x: x[0], selNames))
        return selNames


    #----- universalDialog -----------
    def okDialog(self, title, mess):
        unvDlg.okDialog(self, title, mess)
    def errDialog(self, title, mess):
        unvDlg.errDialog(self, title, mess)
    def inputTextDialog(self, title, message, entryText):
        dialog = inputTextDDialog.dialogApp(title, message, entryText)
        inputText = dialog.show()
        return inputText
    def okCancelDialog(self, title, mess):
        return unvDlg.okCancelDialog(self, title, mess)



#
#  getStlFilesCont
#
def getStlFilesCont(stlFiles):
    """ stlFileの内容（ascii or binary, size etc.)を取得する
    
    Args:
        stlFiles (list) :確認するstlファイル(fullPath)
    Returns:
        stlファイル毎に以下のlistを返す
        stlName (str)           :stlFileのbaseName
        fileFmts (str)          :"ascii" or "bin"
        solidNames (str)        :solid名
        stlSizes (list(float))  :stlファイルのサイズ[x,y,z]"""
    stlConts = []
    for stlFile in stlFiles:
        stlCont = []
        stlName = stlFile.split(os.sep)[-1]
        stlCont.append(stlName)
        if isBinaryStl(stlFile) == False:
            stlCont.append("ascii")
            (solidName, xyzVals) = getStlContFromAscii(stlFile)
        else:
            stlCont.append("bin")
            (solidName, xyzVals) = getStlContFromBin(stlFile)
        stlCont.append(solidName)
        (xVals, yVals, zVals) = xyzVals
        if len(xVals) != 0:
            dx = max(xVals) - min(xVals)
            dy = max(yVals) - min(yVals)
            dz = max(zVals) - min(zVals)
        else:
            dx, dy, dz = 0.0, 0.0, 0.0
        #値を丸める
        dx = getValueLog10(dx)
        dy = getValueLog10(dy)
        dz = getValueLog10(dz)
        stlCont.append([dx, dy, dz])
        stlConts.append(stlCont)
    stlConts.sort()
    return stlConts

#  isBinaryStl
def isBinaryStl(fileName):
    """ binaryファイルかどうかチェック。header(80bytes)を除いた9文字をチェック
    
    Args:
        fileName (str) :stlFile
    Return:
        bool    :True or False"""
    f = open(fileName, "rb"); cont = f.read(100); f.close()
    if len(cont) < 100:
        return False
    ans = False
    for i in range(81, 90):
        if ((ord(" ") <= cont[i] and (cont[i] <= ord("z")) or
            cont[i] == ord("\n") or cont[i] == ord("\r") or cont[i] == ord("\t"))):
            pass
        else:
            ans = True
            break
    return ans

#  getStlContFromAscii
def getStlContFromAscii(fileName):
    """ asciiのstlファイルからsolidNameとxyz座標を取得
    
    Args:
        stlFile (str)   :stlファイル
    Return:
        solidName (str) :solid名
        xyzVals (list)  :[xVlas, yVals, zVals]"""
    f=open(fileName); lines=f.readlines(); f.close()
    #solidName取得
    names = []
    for line in lines:
        if line[:5] == "solid":
            name = " ".join(line.split()[1:])
            names.append(name)
    solidName = ",".join(names)
    #stlSize取得
    xVals=[]; yVals=[]; zVals=[]
    for line in lines:
        if line.find("vertex") >= 0:
            values = line.split()
            xVals.append(float(values[1]))
            yVals.append(float(values[2]))
            zVals.append(float(values[3]))
    xyzVals = [xVals, yVals, zVals]
    return (solidName, xyzVals)

#  getStlContFromBin
def getStlContFromBin(fileName):
    """ binaryのstlファイルからsolidNameとxyz座標を取得
    
    Args:
        stlFile (str)   :stlファイル
    Return:
        solidName (str) :solid名
        xyzVals (list)  :[xVlas, yVals, zVals]"""
    f = open(fileName, "rb"); cont = f.read(); f.close()
    #header部を取得
    i = 80
    header = cont[:i]
    #solidNameを取得
    solidName = ""
    for chara in header:
        #binaryの場合、数値でで大小を比較する
        if ord(" ") <= chara and chara <= ord("z"):
            solidName += chr(chara)
        else:
            break
    #三角形の数
    num = struct.unpack("1i", cont[i:i+4])
    _nTet = num[0]
    i += 4
    #座標を取得
    xVals = []; yVals = []; zVals = []
    while i < len(cont):
        num = struct.unpack("12f", cont[i:i+48])
        #単位ベクトルはskipする
        xVals += [num[3]] + [num[6]] + [num[9]]
        yVals += [num[4]] + [num[7]] + [num[10]]
        zVals += [num[5]] + [num[8]] + [num[11]]
        #最後の2byte分も加えてskip
        i += 48 + 2
    xyzVals = [xVals, yVals, zVals]
    return (solidName, xyzVals)

#  getValueLog10
def getValueLog10(x):
    """ 値を有効数字3桁に丸め、最終桁の「0」を削除する"""
    decimal.getcontext().prec = 3
    xx = str(decimal.Decimal(x) + decimal.Decimal(0))
    if xx == "0":
        xx = "0.0"
    return xx
    # #最終桁の「0」を削除
    # i = len(xx)
    # while i > 0:
    #     i -= 1
    #     if xx[i] != "0":
    #         break
    # xx = xx[:i+1]
    # return xx


if __name__ == "__main__":
    import gettext
    gettext.install("treefoam", localeDir)
    #_ = gettext.gettext

    caseDir = sys.argv[1]
    stlDir = sys.argv[2]
    winApp = windowApp(caseDir, stlDir)
    winApp.main()
